#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>

#include "i2cmaster.h"
#include "nunchuk.h"



#define UBRR_38400 12
/* INIT USART (RS-232)  */

void USART_init( uint16_t ubrr )
{
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: Off
	// USART Transmitter: On
	// USART0 Mode: Asynchronous
	// USART Baud Rate: 38400
	UCSR0A=0x00;
	UCSR0B=0x08;
	UCSR0C=0x06;
	UBRR0=ubrr;
}


char buff[64];

/* SEND A STRING TO THE RS-232 */
void USART_tx_string( char *data )
{
   while ((*data != '\0'))
   {
      while (!(UCSR0A & (1 <<UDRE0)));
      UDR0 = *data;
      data++;
   }
}

void USART_tx( char c )
{
  while (!(UCSR0A & (1 <<UDRE0)));
  UDR0 = c;
}

/*
  .

0.    0.
0.  
0.    +1.2 +/- 50 (   )

1.  
1.   +0.3

-   1.5

2.  
2.    +1.2 +/- 50 (   )

3.  
3.   +0.3

-   3

4.  
4.    +1.2 +/- 50 (   )

5.  
5.    +0.3

-   4.5

6.  
6.    24.7   ( 20  ,    .)

-   - 24.7

7.  
7.    +0.3

-   - 25.0

8. -> 0


*/

#define PPM_DDR DDRD
#define PPM_PORT PORTD
#define PPM_PIN PD7

#define TIMER_MULT 80

volatile uint8_t p_state = 0;
volatile int8_t channels[4];
volatile uint16_t total_len = 2000;

#define ON PPM_PORT |= _BV(PPM_PIN);
#define OFF PPM_PORT &= ~_BV(PPM_PIN);

ISR(TIMER1_COMPA_vect) {
	if (p_state == 0) {
		TCNT1 = 0;
		ON;
		OCR1A = TCNT1 + (120 + channels[0])*TIMER_MULT;
		p_state = 1;
	} 
	else if (p_state  == 1) {
		OFF;
		OCR1A = TCNT1 + 30*TIMER_MULT;
		p_state = 2;
	}
	else if (p_state  == 2) {
		ON;
		OCR1A = TCNT1 + (120 + channels[1])*TIMER_MULT;
		p_state = 3;
	}
	else if (p_state  == 3) {
		OFF;
		OCR1A = TCNT1 + 30*TIMER_MULT;
		p_state = 4;
	}
	else if (p_state  == 4) {
		ON;
		OCR1A = TCNT1 + (120 + channels[2])*TIMER_MULT;
		p_state = 5;
	}
	else if (p_state  == 5) {
		OFF;
		OCR1A = TCNT1 + 30*TIMER_MULT;
		p_state = 6;
	}
	else if (p_state  == 6) {
		ON;
		OCR1A = TCNT1 + (120 + channels[3])*TIMER_MULT;
		p_state = 7;
	}
	else if (p_state  == 7) {
		OFF;
		OCR1A = TCNT1 + 30*TIMER_MULT;
		p_state = 8;
	}
	else if (p_state  == 8) {
		ON;
		OCR1A = total_len*TIMER_MULT;
		p_state = 9;
	}
	else if (p_state  == 9) {
		OFF;
		OCR1A = TCNT1 + 30*TIMER_MULT;
		p_state = 0;
	}



}

/*
ISR(PCINT0_vect)
{
	if (p_state == -1)
	{
		if (PINB & _BV(PB0))
		{
			ON;
		}
		else
		{
			OFF;
		}
	}
}
*/

inline void TIMER1_init()
{
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 1000.000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// Noise Canceler: Off
	// Input Capture on Raising Edge
	// Timer1 Overflow Interrupt: On
	// Input Capture Interrupt: On
	// Compare A Match Interrupt: On
	// Compare B Match Interrupt: Off

	//TCCR1A=0x00;
	TCCR1B= _BV(CS10) | _BV(ICNC1);
	//TCNT1H=0x00;
	//TCNT1L=0x00;
	//ICR1H=0x00;
	//ICR1L=0x00;
	//OCR1AH=0x00;
	//OCR1AL=0x00;
	//OCR1BH=0x00;
	//OCR1BL=0x00;

	// Timer/Counter 1 Interrupt(s) initialization
	//TIMSK1= _BV(ICIE1) | _BV(TOIE1);
	TIMSK1= _BV(OCIE1A);

}

inline void PCINT_init()
{
	PCICR |= _BV(PCIE0);
	PCIFR |= _BV(PCIF0);
	PCMSK0 = _BV(PCINT0);
}


int main(void)
{
	unsigned char res;
	uint8_t buffer_data[sizeof(nunchuk_data_t)];
	nunchuk_data_t *nunchuk_data = (nunchuk_data_t *)buffer_data;

	uint8_t buffer_axis[sizeof(nunchuk_data_t)];
	nunchuk_axis_t *nunchuk_axis = (nunchuk_axis_t *)buffer_axis;


    USART_init(UBRR_38400);  // Initialize the USART
    _delay_ms(10);

    USART_tx_string("Connected!\n\r");    // we're alive!

    sei(); // enable interrupts

	i2c_init();
	_delay_ms(10);

	PPM_DDR |= _BV(PPM_PIN);
	
	TIMER1_init();
	//PCINT_init();

	nunchuk_init();

	USART_tx_string("nunchuk init ok\n\r");

	while (1) {

		nunchuk_read(buffer_data);
        nunchuk_calc_axis(buffer_data, buffer_axis);


//		if (! nunchuk_axis->but_c)
//		{
//			//channels[0] = 0;
//			//channels[1] = 1;
//			//channels[2] = 2;
//			p_state = -1;
//
//			//memset(buff, 0, sizeof(buff));
//			//sprintf(buff, "passthru - c:%d, z:%d\n\r", nunchuk_axis->but_c, nunchuk_axis->but_z);
//			//USART_tx_string(buff);
//			//_delay_ms(30);
//
//		}
//		else {

			// chan 0, Y->, 5 . 
			if (nunchuk_axis->y <= 0)
			{
				// -95 - 0 => (-16 - 0)*3
				if (nunchuk_axis->y <= -95)
					nunchuk_axis->y = -95;

				channels[0] = ((float)nunchuk_axis->y * 16.0 / 95.0)* 3;
			} else 
			{
				// 0 - 129 => (0 - 16)*3
				if (nunchuk_axis->y > 129)
					nunchuk_axis->y = 129;
				
				channels[0] = ((float)nunchuk_axis->y * 16.0 / 129.0)* 3;
			}


			// chan 1, pitch->elevator, 5 . 
			if (nunchuk_axis->pitch <= 0)
			{
				// -70 - 0 => (-16 - 0)*3
				if (nunchuk_axis->pitch <= -45)
					nunchuk_axis->pitch = -45;

				channels[1] = ((float)nunchuk_axis->pitch * 16.0 / 45)* 3;
			} else 
			{
				// 0 - 70 => (0 - 16)*3
				if (nunchuk_axis->pitch > 45)
					nunchuk_axis->pitch = 45;
				
				channels[1] = ((float)nunchuk_axis->pitch * 16.0 / 45)* 3;
			}
		

			// chan 2, roll->rudder, 5 . 
			if (nunchuk_axis->roll <= 0)
			{
				// -90 - 0 => (-16 - 0)*3
				if (nunchuk_axis->roll <= -45)
					nunchuk_axis->roll = -45;

				channels[2] = ((float)nunchuk_axis->roll * 16.0 / 45)* 3;
			} else 
			{
				// 0 - 90 => (0 - 16)*3
				if (nunchuk_axis->roll > 45)
					nunchuk_axis->roll = 45;
				
				channels[2] = ((float)nunchuk_axis->roll * 16.0 / 45)* 3;
			}

			
			// chan 3 x -> ch3, 5 
			if (nunchuk_axis->x <= 0)
			{
				// -95 - 0 => (-16 - 0)*3
				if (nunchuk_axis->x <= -95)
					nunchuk_axis->x = -95;

				channels[3] = ((float)nunchuk_axis->x * 16.0 / 95.0)* 3;
			} else 
			{
				// 0 - 129 => (0 - 16)*3
				if (nunchuk_axis->x > 129)
					nunchuk_axis->x = 129;
				
				channels[3] = ((float)nunchuk_axis->x * 16.0 / 129.0)* 3;
			}


			//if (p_state == -1)
			//	p_state = 0;


			/*
			memset(buff, 0, sizeof(buff));
			sprintf(buff, "Y %3d -> Ch0 %3d ; PITCH %3d -> Ch1 %3d ; ROLL %3d -> Ch2 %3d\n\r", nunchuk_axis->y, channels[0], nunchuk_axis->pitch, channels[1], nunchuk_axis->roll, channels[2]);
			USART_tx_string(buff);
			*/


//		}


		/*
        memset(buff, 0, sizeof(buff));
		//sprintf(buff, "struct: %x %x %x %x\n\r", buffer[0], buffer[1], buffer[2], buffer[3]);
        sprintf(buff, "JOY X:%3d Y:%3d, BUTT: Z:%d C:%d, ROLL: %4d PITCH: %4d\n\r", nunchuk_axis->x, nunchuk_axis->y, nunchuk_data->but_z, nunchuk_data->but_c, nunchuk_axis->roll, nunchuk_axis->pitch);
		USART_tx_string(buff);

        memset(buff, 0, sizeof(buff));
        sprintf(buff, "channels 1:%3d 2:%3d, 3:%3d\n\r", channels[0], channels[1], channels[2]);
		USART_tx_string(buff);
		*/



		/*
        pos = 0;
        t_state = 0;
        TIMER1_init();

		while (t_state != 2);

        for (uint8_t i=0; i<pos; i++)
        {
            memset(buff, 0, sizeof(buff));
            sprintf(buff, "%u ", intervals[i]/20);
    		USART_tx_string(buff);

        }
        USART_tx_string("\n\r");
		*/


		_delay_ms(10);

	}

}
